package net.codinux.banking.fints.model

import kotlinx.datetime.LocalDate
import net.codinux.banking.fints.extensions.UnixEpochStart


open class AccountTransaction(
    val account: AccountData,
    val amount: Money,
    val reference: String?, // that was also new to me that reference may is null

    val bookingDate: LocalDate,
    val valueDate: LocalDate,

    /**
     * Name des Überweisenden oder Zahlungsempfängers
     */
    val otherPartyName: String?,
    /**
     * BIC des Überweisenden / Zahlungsempfängers
     */
    val otherPartyBankId: String?,
    /**
     * IBAN des Überweisenden oder Zahlungsempfängers
     */
    val otherPartyAccountId: String?,

    /**
     * Buchungstext, z. B. DAUERAUFTRAG, BARGELDAUSZAHLUNG, ONLINE-UEBERWEISUNG, FOLGELASTSCHRIFT, ...
     */
    val postingText: String?,

    val openingBalance: Money?,
    val closingBalance: Money?,

    /**
     * Auszugsnummer
     */
    val statementNumber: Int,
    /**
     * Blattnummer
     */
    val sheetNumber: Int?,

    /**
     * Kundenreferenz.
     */
    val customerReference: String?,
    /**
     * Bankreferenz
     */
    val bankReference: String?,
    /**
     * Währungsart und Umsatzbetrag in Ursprungswährung
     */
    val furtherInformation: String?,


    /*      Remittance information      */

    val endToEndReference: String?,
    val mandateReference: String?,
    val creditorIdentifier: String?,
    val originatorsIdentificationCode: String?,

    /**
     * Summe aus Auslagenersatz und Bearbeitungsprovision bei einer nationalen Rücklastschrift
     * sowie optionalem Zinsausgleich.
     */
    val compensationAmount: String?,
    /**
     * Betrag der ursprünglichen Lastschrift
     */
    val originalAmount: String?,
    /**
     * Abweichender Überweisender oder Zahlungsempfänger
     */
    val deviantOriginator: String?,
    /**
     * Abweichender Zahlungsempfänger oder Zahlungspflichtiger
     */
    val deviantRecipient: String?,
    val referenceWithNoSpecialType: String?,

    /**
     * Primanoten-Nr.
     */
    val journalNumber: String?,
    /**
     * Bei R-Transaktionen siehe Tabelle der
     * SEPA-Rückgabecodes, bei SEPALastschriften siehe optionale Belegung
     * bei GVC 104 und GVC 105 (GVC = Geschäftsvorfallcode)
     */
    val textKeyAddition: String?,

    /**
     * Referenznummer, die vom Sender als eindeutige Kennung für die Nachricht vergeben wurde
     * (z.B. als Referenz auf stornierte Nachrichten).
     */
    val orderReferenceNumber: String?,
    /**
     * Bezugsreferenz
     */
    val referenceNumber: String?,

    /**
     * Storno, ob die Buchung storniert wurde(?).
     * Aus:
     * „RC“ = Storno Haben
     * „RD“ = Storno Soll
     */
    val isReversal: Boolean

    /**     7.1.7.6 RemittanceInformation
     * CreditorReferenceInformation  Referenzinformationen, die vom Zahlungsempfänger für die Überweisung vorgegeben wurden
     * Creditor         Zahlungsempfängerseite
     * Debtor           Zahlerseite
     * UltimateDebtor   Abweichender Zahler
     *ReferenceNumber   (Steuer)referenzinformationen
     */

    /**     7.1.7.1 References
     * Name XML-Tag Kardinalität
     * Definition Typ DKBelegungsregel
     * 6 MessageIdentifica
     * tion
     * <MsgId> [0..1] Message-Id <MsgId>
     * aus der referenzierten
     * pain-Nachricht
     * Max35Text
     * 6 AccountServicerR
     * eference
     * <AcctSvcrR
     * ef>
     * [0..1] Bankreferenz Max35Text
     * 6 PaymentInformati
     * onIdentification
     * <PmtInfId> [0..1] Payment Information Id
     * aus der referenzierten
     * pain-Nachricht
     * Max35Text
     * 6 InstructionIdentific
     * ation
     * <InstrId> [0..1] Referenz der den
     * Auftrag übermittelnden
     * Partei
     * Max35Text
     * 6 EndToEndIdentifi
     * cation
     * <EndToEndI
     * d>
     * [0..1] eindeutige Referenz des
     * Überweisenden
     * Max35Text CHK: Konstante
     * „SCHECK-NR. “,
     * gefolgt von der
     * Schecknummer
     * (entspricht dem
     * Inhalt der
     * EndToEndId des
     * entsprechenden
     * Scheckumsatzes
     * )
     * 6 UETR <UETR> [0..1] Universal eindeutige
     * Referenz einer
     * Zahlungstransaktion
     * UUIDv4Identifi
     * er
     * 6 TransactionIdentif
     * ication
     * <TxId> [0..1] Id des erstbeteiligten
     * Kreditinstituts (G1)
     * Max35Text CHK: ISE/BSEReferenz in den
     * ersten 12 Stellen
     * WP: hier kann
     * die Nummer der
     * separaten WPAbrechung
     * angegeben
     * werden
     * 6 MandateIdentifica
     * tion
     * <MndtId> [0..1] Mandatsreferenz Max35Text
     * 6 ChequeNumber <ChqNb> [0..1] Schecknummer Max35Text CHK:
     * Schecknummer
     * 6 ClearingSystemR
     * eference
     * <ClrSysRef> [0..1] Referenz eines
     * Clearingsystems
     * Max35Text
     * 6 AccountOwnerTra
     * nsactionIdentificat
     * ion
     * <AcctOwnrT
     * xId>
     * [0..1] Eindeutige
     * Identifizierung der WPTransaktion, wie sie dem
     * Depotinhaber oder
     * dessen Depotbetreuer
     * bekannt ist
     * Max35Text
     * 6 AccountServicerT
     * ransactionIdentific
     * ation
     * <AcctSvcrTx
     * Id>
     * [0..1] Eindeutige
     * Identifizierung der
     * Wertpapiertransaktion,
     * wie sie dem
     * kontoführenden Institut
     * bekannt ist
     * Max35Text
     * 6 MarketInfrastructu
     * reTransactionIden
     * tification
     * <MktInfrstrct
     * rTxId>
     * [0..1] Identifizierung einer WPTransaktion durch eine
     * Marktinfrastruktur/Zentral
     * verwahrer, zum Beispiel
     * TARGET2-Securities.
     * Max35Text
     * 6 ProcessingIdentifi
     * cation
     * <PrcgId> [0..1] Identfizierung der WPTransaktion durch
     * Pozessor, wenn
     * abweichend von den bei
     * Depotinhaber/Depotbetr
     * euer und
     * Marktinfrastruktur
     * angegebener Id
     * Max35Text
     */

    /**     7.1.7.4 Related Parties
     * Name XML-Tag Kardinalität
     * Definition Typ DKBelegungsregel
     * 6 InitiatingParty <InitgPty> [0..1] Anweisende Partei Siehe 7.1.8.6
     * 6 Debtor <Dbtr> [0..1] Überweisender /
     * Zahlungspflichtiger
     * Siehe 7.1.8.6
     * 6 DebtorAccount <DbtrAcct> [0..1] Konto des
     * Überweisenden /
     * Zahlungspflichtigen
     * Siehe 7.1.8.4
     * 6 UltimateDebtor <UltmtDbtr> [0..1] Zahlungspflichtiger
     * sofern abweichend vom
     * Kontoinhaber
     * Siehe 7.1.8.6
     * 6 Creditor <Cdtr> [0..1] Begünstigter /
     * Zahlungsempfänger
     * Siehe 7.1.8.6 Bei SEPALastschrift: Hier
     * ist im Zweig
     * <Pty> unter
     * <Id><PrvtId>
     * <Othr> der
     * Creditor-Identifier
     * anzugeben (wie
     * in pain.008) –
     * siehe auch direkt
     * unter 7.1.8.2
     * (=<Id>)
     * 6 CreditorAccount <CdtrAcct> [0..1] Konto des Begünstigten
     * / Zahlungsempfängers
     * Siehe 7.1.8.4
     * 6 UltimateCreditor <UltmtCdtr> [0..1] Zahlungsempfänger
     * sofern abweichend vom
     * Kontoinhaber
     * Siehe 7.1.8.6
     */
) {

    // for object deserializers
    internal constructor() : this(AccountData(), Money(Amount.Zero, ""), "", UnixEpochStart, UnixEpochStart, null, null, null, null)

    constructor(account: AccountData, amount: Money, unparsedReference: String, bookingDate: LocalDate, valueDate: LocalDate, otherPartyName: String?, otherPartyBankId: String?, otherPartyAccountId: String?, postingText: String? = null)
        : this(account, amount, unparsedReference, bookingDate, valueDate, otherPartyName, otherPartyBankId, otherPartyAccountId, postingText,
        null, null, 0, null,
        null, null, null, null, null, null, null, null, null, null, null,
        "", null, null, "", null, false)


    open val showOtherPartyName: Boolean
        get() = otherPartyName.isNullOrBlank() == false /* && type != "ENTGELTABSCHLUSS" && type != "AUSZAHLUNG" */ // TODO


    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other !is AccountTransaction) return false

        if (account != other.account) return false
        if (amount != other.amount) return false
        if (reference != other.reference) return false
        if (bookingDate != other.bookingDate) return false
        if (otherPartyName != other.otherPartyName) return false
        if (otherPartyBankId != other.otherPartyBankId) return false
        if (otherPartyAccountId != other.otherPartyAccountId) return false
        if (postingText != other.postingText) return false
        if (valueDate != other.valueDate) return false

        return true
    }

    override fun hashCode(): Int {
        var result = account.hashCode()
        result = 31 * result + amount.hashCode()
        result = 31 * result + reference.hashCode()
        result = 31 * result + bookingDate.hashCode()
        result = 31 * result + (otherPartyName?.hashCode() ?: 0)
        result = 31 * result + (otherPartyBankId?.hashCode() ?: 0)
        result = 31 * result + (otherPartyAccountId?.hashCode() ?: 0)
        result = 31 * result + (postingText?.hashCode() ?: 0)
        result = 31 * result + valueDate.hashCode()
        return result
    }


    override fun toString(): String {
        return "$valueDate $amount $otherPartyName: $reference"
    }

}