package net.codinux.banking.fints.model.parameter

import net.codinux.banking.fints.messages.datenelemente.abgeleiteteformate.Laenderkennzeichen
import net.codinux.banking.fints.messages.datenelemente.implementierte.BPDVersion
import net.codinux.banking.fints.messages.datenelemente.implementierte.Dialogsprache
import net.codinux.banking.fints.messages.datenelemente.implementierte.HbciVersion
import net.codinux.banking.fints.messages.segmente.id.ISegmentId
import net.codinux.banking.fints.model.TanMethod
import net.codinux.banking.fints.response.segments.JobParameters
import net.codinux.banking.fints.response.segments.PinInfo

/**
 * Die basalen Parameter / (Konfigurations-)Werte einer Bank. Für alle Kunden dieser Bank gleich.
 * 
 * TODO: noch besseren Namen finden
 *
 * 1. Protocol Communication Data
 *
 *     ProtocolContext
 *     CommunicationInfo
 *     ProtocolConnectionData
 *     TransportData
 *     ProtocolMetadata
 *     NetworkState
 *     ConnectionDetails
 *
 * 2. User Data for UI
 *
 *     UserProfile
 *     UserDisplayData
 *     UserViewModel
 *     UserInfo
 *     UIUserData
 *     UserPresentation
 *     UserDisplayModel
 */
open class BankParameter(
    var bankCode: String,
    var bankName: String,
    var bic: String,
    var finTs3ServerAddress: String,

    var bpdVersion: Int = BPDVersion.VersionNotReceivedYet,
    var supportedLanguages: List<Dialogsprache> = listOf(),
    var countryCode: Int = Laenderkennzeichen.Germany,

    var supportedTanMethods: List<TanMethod> = listOf(),

    /**
     * Maximale Anzahl an Geschäftsvorfallsarten, die pro Nachricht zulässig ist.
     * Der Wert ‚0’ gibt an, dass keine Restriktionen bzgl. der Anzahl an Geschäftsvorfallsarten bestehen.
     */
    var countMaxJobsPerMessage: Int = 0,

    var supportedHbciVersions: List<HbciVersion> = listOf(),
    var supportedJobs: List<JobParameters> = listOf()
) {
    

    /**
     * Some banks use a special bank code for online banking that doesn't match bank's bank code, e. g. Hypo Vereinsbank
     */
    // rename to bankCodeForFinTS ?
    val bankCodeForOnlineBanking: String
        get() {
            // for UniCredit / HypVereinsbank for online banking '70020270' has to be used as bank code
            if (bankName.contains("unicredit", true) || bic.startsWith("HYVEDE")) {
                return "70020270"
            }

            return bankCode
        }


    open var jobsRequiringTan: Set<String> = emptySet()
        protected set

    open var pinInfo: PinInfo? = null
        set(value) {
            field = value
            // TODO: in case of null: actually in this case it's not allowed to execute job via PIN/TAN at all
            jobsRequiringTan = value?.jobTanConfiguration.orEmpty().filter { it.tanRequired }.map { it.segmentId }.toSet()
        }

    open fun doesJobRequireTan(segmentId: ISegmentId): Boolean = doesJobRequireTan(segmentId.id)

    open fun doesJobRequireTan(segmentId: String): Boolean =
        jobsRequiringTan.contains(segmentId)


    override fun toString(): String {
        return "$bankCode $bankName"
    }
    
}