package net.codinux.banking.fints.model.parameter

import net.codinux.banking.fints.messages.datenelemente.implementierte.*
import net.codinux.banking.fints.messages.datenelemente.implementierte.tan.TanMedium
import net.codinux.banking.fints.model.AccountData
import net.codinux.banking.fints.model.BankData.Companion.TanMethodNotSelected
import net.codinux.banking.fints.model.TanMethod
import net.codinux.banking.fints.response.segments.ChangeTanMediaParameters
import net.codinux.banking.fints.response.segments.PinInfo

/**
 * The basic data, e.g. her allowed TAN methods and Jobs, of a customer at a specific bank.
 *
 * Im Gegensatz zu [BankParameter] weiss ich noch nicht, wie sinnvoll diese Klasse ist. Halt irgendwo die FinTS spezifischen
 * Daten (die fuer die UI voellig egal sind) wie die customerSystemId, allowedJobs pro account und UPD Version zu speichern
 *
 * TODO: noch besseren Namen finden
 */
open class CustomerParameter(
    open var customerId: String,
    open var pin: String,

    open var userId: String = customerId,
    open var customerName: String = "",
    open var updVersion: Int = UPDVersion.VersionNotReceivedYet,

    open var tanMethodsAvailableForUser: List<TanMethod> = listOf(),
    open var selectedTanMethod: TanMethod = TanMethodNotSelected,
    open var tanMedia: List<TanMedium> = listOf(),
    open var selectedTanMedium: TanMedium? = null,
    open var changeTanMediumParameters: ChangeTanMediaParameters? = null,
    open var pinInfo: PinInfo? = null,

    open var selectedLanguage: Dialogsprache = Dialogsprache.Default,
    open var customerSystemId: String = KundensystemID.Anonymous,
    open var customerSystemStatus: KundensystemStatusWerte = KundensystemStatus.SynchronizingCustomerSystemId,
) {

    // here we actually need per account only the (full) account identifier, the allowedJob(Name)s, and
    // may isAccountTypeSupportedByApplication and the supported features
    val accounts = mutableListOf<AccountData>()


    open val isTanMethodSelected: Boolean
        get() = selectedTanMethod != TanMethodNotSelected
}